﻿Imports Infragistics.Win.UltraWinTree
Imports UltraTreeDDSample.UltraTreeDrawFilter

Public Class Form1
    Inherits System.Windows.Forms.Form

    'Create a new instance of the drawfilter class to handle drawing the drop highlight/drop lines
    Private WithEvents utvDrawFilter As New UltraTreeDrawFilter()

    Private mDragNodes As SelectedNodesCollection

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        'Attach the DrawFilter to the tree
        UltraTree1.DrawFilter = utvDrawFilter

        'Populate the tree with data
        LoadTree()

        UltraTree1.Override.SelectionType = SelectType.ExtendedAutoDrag

    End Sub

    'This proc populates the tree with continents and countries.
    'They are all set up on the root level so you can have
    'fun putting the countries under the right continent. :)
    Private Sub LoadTree()
        Dim ContinentNode As UltraTreeNode
        Dim CountryNode As UltraTreeNode

        With UltraTree1
            .Appearances.Add("DropHighLightAppearance")
            With .Appearances("DropHighLightAppearance")
                .BackColor = Color.Cyan
            End With

            ContinentNode = .Nodes.Add("Continent:North America", "North America")
            ContinentNode = .Nodes.Add("Continent:South America", "South America")
            ContinentNode = .Nodes.Add("Continent:Antarctica", "Antarctica")
            ContinentNode = .Nodes.Add("Continent:Europe", "Europe")
            ContinentNode = .Nodes.Add("Continent:Asia", "Asia")
            ContinentNode = .Nodes.Add("Continent:Africa", "Africa")
            ContinentNode = .Nodes.Add("Continent:Australia", "Australia")

            CountryNode = .Nodes.Add("Australia")
            CountryNode.Tag = "Australia"
            CountryNode = .Nodes.Add("Austria")
            CountryNode.Tag = "Europe"
          
        End With

        With UltraTree2
            For i As Integer = 0 To 5
                .Nodes.Add("Root Node " & i.ToString)
                .Tag = i

                For c As Integer = 0 To 5
                    .Nodes(i).Nodes.Add("Child Node " & c.ToString & "(RN" & i.ToString & "0)")
                    .Tag = i.ToString & c.ToString
                Next
            Next
        End With

        With UltraTree3
            For i As Integer = 0 To 11
                .Nodes.Add("Root Node " & i.ToString)
                .Tag = i

                For c As Integer = 6 To 11
                    .Nodes(i).Nodes.Add("Child Node " & c.ToString & "(RN" & i.ToString & "0)")
                    .Tag = i.ToString & c.ToString
                Next
            Next


        End With
    End Sub

    Private Sub UltraTree1_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles UltraTree1.DragDrop
        'A dummy node variable used for various things
        Dim aNode As UltraTreeNode
        'The SelectedNodes which will be dropped
        Dim SelectedNodes As SelectedNodesCollection
        'The Node to Drop On
        Dim DropNode As Infragistics.Win.UltraWinTree.UltraTreeNode
        'An integer used for loops
        Dim i As Integer

        'Set the DropNode
        DropNode = utvDrawFilter.DropHightLightNode

        'Get the Data and put it into a SelectedNodes collection,
        'then clone it and work with the clone
        'These are the nodes that are being dragged and dropped
        SelectedNodes = e.Data.GetData(GetType(SelectedNodesCollection))
        SelectedNodes = SelectedNodes.Clone()

        'Sort the selected nodes into their visible position. 
        'This is done so that they stay in the same order when
        'they are repositioned. 
        SelectedNodes.SortByPosition()

        'Determine where we are dropping based on the current
        'DropLinePosition of the DrawFilter
        Select Case utvDrawFilter.DropLinePosition
            Case DropLinePositionEnum.OnNode 'Drop ON the node
                'Loop through the SelectedNodes and reposition
                'them to the node that was dropped on.
                'Note that the DrawFilter keeps track of what
                'node the mouse is over, so we can just use
                'DropHighLightNode as the drop target. 
                For i = 0 To SelectedNodes.Count - 1
                    aNode = SelectedNodes(i)
                    aNode.Reposition(DropNode.Nodes)
                Next
            Case DropLinePositionEnum.BelowNode 'Drop Below the node
                For i = 0 To SelectedNodes.Count - 1
                    aNode = SelectedNodes(i)
                    aNode.Reposition(DropNode, NodePosition.Next)
                    'Change the DropNode to the node that was
                    'just repositioned so that the next 
                    'added node goes below it. 
                    DropNode = aNode
                Next
            Case DropLinePositionEnum.AboveNode
                'The nodes are added backwards to they
                'retian their original order
                For i = 0 To SelectedNodes.Count - 1
                    aNode = SelectedNodes(i)
                    aNode.Reposition(DropNode, NodePosition.Previous)
                Next
        End Select

        'After the drop is complete, erase the current drop
        'highlight. 
        utvDrawFilter.ClearDropHighlight()

    End Sub

    Private Sub UltraTree1_DragOver(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles UltraTree1.DragOver
        'A dummy node variable used to hold nodes for various 
        'things
        Dim aNode As UltraTreeNode
        'The Point that the mouse cursor is on, in Tree coords. 
        'This event passes X and Y in form coords. 
        Dim PointInTree As Point

        'Get the position of the mouse in the tree, as opposed
        'to form coords
        PointInTree = UltraTree1.PointToClient(New Point(e.X, e.Y))

        'Get the node the mouse is over
        aNode = UltraTree1.GetNodeFromPoint(PointInTree)

        'Make sure the mouse is over a node
        If aNode Is Nothing Then
            'The Mouse is not over a node
            'Do not allow dropping here
            e.Effect = DragDropEffects.None
            'Erase any DropHighlight
            utvDrawFilter.ClearDropHighlight()
            'Exit stage left
            Exit Sub
        End If

        '    Don't let continent nodes be dropped onto other continent nodes
        If Me.IsContinentNode(aNode) And Me.IsContinentNodeSelected(Me.UltraTree1) Then

            If PointInTree.Y > (aNode.Bounds.Top + 2) AndAlso _
             PointInTree.Y < (aNode.Bounds.Bottom - 2) Then
                e.Effect = DragDropEffects.None
                utvDrawFilter.ClearDropHighlight()
                Return

            End If
        End If

        'Check to see if we are dropping onto a node who's
        'parent (grandparent, etc) is selected.
        'This is to prevent the user from dropping a node onto
        'one of it's own descendents. 
        If IsAnyParentSelected(aNode) Then
            'Mouse is over a node whose parent is selected
            'Do not allow the drop
            e.Effect = DragDropEffects.None
            'Clear the DropHighlight
            utvDrawFilter.ClearDropHighlight()
            'Exit stage left
            Exit Sub
        End If

        'If we've reached this point, it's okay to drop on this node
        'Tell the DrawFilter where we are by calling SetDropHighlightNode
        utvDrawFilter.SetDropHighlightNode(aNode, PointInTree)
        'Allow Dropping here. 
        e.Effect = DragDropEffects.Move

    End Sub

    Private Sub UltraTree1_SelectionDragStart(ByVal sender As Object, ByVal e As System.EventArgs) Handles UltraTree1.SelectionDragStart

        'Start the Drag Drop
        UltraTree1.DoDragDrop(UltraTree1.SelectedNodes, DragDropEffects.Move)

    End Sub

    'Proc to check whether a node is a continent or not. 
    'This is necessary because we only want continents to be 
    'parent nodes. We don't want countries to contain other
    'countries
    Private Function IsContinentNode(ByVal Node As UltraTreeNode) As Boolean
        'The Key of the node
        Dim NodeKey As String
        'The beginning of the key
        Dim ParsedNodeKey() As String

        'Get the key of the node
        NodeKey = Node.Key
        'Parse it out by colon
        ParsedNodeKey = Split(NodeKey, ":")

        'If the beginning of the key says Continent, then 
        'we know it's a continent node. 
        Return (ParsedNodeKey(0) = "Continent")
    End Function

    ' Returns whether any of the currently selected nodes in the
    ' specified UltraTree control represent continents.
    Private Function IsContinentNodeSelected(ByVal tree As UltraTree) As Boolean

        Dim selectedNode As UltraTreeNode
        For Each selectedNode In tree.SelectedNodes
            If (Me.IsContinentNode(selectedNode)) Then Return True
        Next

        Return False
    End Function

    'Test to see if we want to continue dragging
    Private Sub UltraTree1_QueryContinueDrag(ByVal sender As Object, ByVal e As System.Windows.Forms.QueryContinueDragEventArgs) Handles UltraTree1.QueryContinueDrag
        'Did the user press escape? 
        If e.EscapePressed Then
            'User pressed escape
            'Cancel the Drag
            e.Action = DragAction.Cancel
            'Clear the Drop highlight, since we are no longer
            'dragging
            utvDrawFilter.ClearDropHighlight()
        End If
    End Sub

    'Occassionally, the DrawFilter will let us know that the 
    'control needs to be invalidated. 
    Private Sub UltraTree_DropHightLight_DrawFilter_Invalidate(ByVal sender As Object, ByVal e As System.EventArgs) Handles utvDrawFilter.Invalidate
        'Any time the drophighlight changes, the control needs 
        'to know that it has to repaint. 
        'It would be more efficient to only invalidate the area
        'that needs it, but this works and is very clean.
        UltraTree1.Invalidate()
    End Sub

    'This event is fired by the DrawFilter to let us determine
    'what kinds of drops we want to allow on any particular node

    Private Sub UltraTree_DropHightLight_DrawFilter_QueryStateAllowedForNode(ByVal sender As Object, ByVal e As QueryStateAllowedForNodeEventArgs) Handles utvDrawFilter.QueryStateAllowedForNode
        'Check to see if this is a continent node. 
        If Not IsContinentNode(e.Node) Then
            'This is not a continent
            'Allow users to drop above or below this node - but not on
            'it, because countries don't have child countries
            e.StatesAllowed = DropLinePositionEnum.AboveNode Or DropLinePositionEnum.BelowNode
            'Since we can only drop above or below this node, 
            'we don't want a middle section. So we set the 
            'sensitivity to half the height of the node
            'This means the DrawFilter will respond to the top half
            'bottom half of the node, but not the middle. 
            utvDrawFilter.EdgeSensitivity = e.Node.Bounds.Height / 2
        Else
            If e.Node.Selected Then
                'This is a selected Continent node. 
                'Since it is selected, we don't want to allow
                'dropping ON this node. But we can allow the
                'the user to drop above or below it. 
                e.StatesAllowed = DropLinePositionEnum.AboveNode Or DropLinePositionEnum.BelowNode
                utvDrawFilter.EdgeSensitivity = e.Node.Bounds.Height / 2
            Else
                'This is a continent node and is not selected
                'We can allow dropping here above, below, or on this
                'node. Since the StatesAllow defaults to All, we don't 
                'need to change it. 
                'We set the EdgeSensitivity to 1/3 so that the 
                'Drawfilter will respond at the top, bottom, or 
                'middle of the node. 
                utvDrawFilter.EdgeSensitivity = e.Node.Bounds.Height / 3
            End If
        End If
    End Sub

    'Walks up the parent chain for a node to determine if any
    'of it's parent nodes are selected
    Private Function IsAnyParentSelected(ByVal Node As UltraTreeNode) As Boolean
        Dim ParentNode As UltraTreeNode
        Dim ReturnValue As Boolean = False

        ParentNode = Node.Parent
        Do Until ParentNode Is Nothing
            If ParentNode.Selected Then
                ReturnValue = True
                Exit Do
            Else
                ParentNode = ParentNode.Parent
            End If
        Loop

        Return ReturnValue
    End Function

    'Fires when the user drags outside the control. 
    Private Sub UltraTree1_DragLeave(ByVal sender As Object, ByVal e As System.EventArgs) Handles UltraTree1.DragLeave
        'When the mouse goes outside the control, clear the 
        'drophighlight. 
        'Since the DropHighlight is cleared when the 
        'mouse is not over a node, anyway, 
        'this is probably not needed
        'But, just in case the user goes from a node directly
        'off the control...
        utvDrawFilter.ClearDropHighlight()
    End Sub

    Private Sub UltraTree1_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles UltraTree1.DragEnter

        If e.Data.GetDataPresent(GetType(SelectedNodesCollection)) Then
            If mDragNodes Is Nothing Then
                e.Effect = DragDropEffects.Move
            Else
                e.Effect = DragDropEffects.Copy
            End If
        Else
            e.Effect = DragDropEffects.None
        End If

    End Sub

    Private Sub UltraTree2_SelectionDragStart(ByVal sender As Object, ByVal e As System.EventArgs) Handles UltraTree2.SelectionDragStart

        mDragNodes = UltraTree2.SelectedNodes


        Dim text As String = mDragNodes.Tag
        For Each node As UltraTreeNode In UltraTree2.SelectedNodes
            UltraTree1.Nodes.Add(node.Clone())

            node.Enabled = False
        Next

        'UltraTree2.DoDragDrop(mDragNodes, DragDropEffects.Copy)
        'UltraTree1.Nodes.Add(text1)
    End Sub

    Private Sub UltraTree3_SelectionDragStart(ByVal sender As Object, ByVal e As System.EventArgs) Handles UltraTree3.SelectionDragStart

        mDragNodes = UltraTree3.SelectedNodes
        For Each node As UltraTreeNode In UltraTree3.SelectedNodes
            UltraTree1.Nodes.Add(node.Clone())
            node.Enabled = False
        Next

        'UltraTree2.DoDragDrop(mDragNodes, DragDropEffects.Copy)

    End Sub

    Private Sub UltraTree2_DragDrop(sender As System.Object, e As System.Windows.Forms.DragEventArgs) Handles UltraTree2.DragDrop
        For Each node As UltraTreeNode In UltraTree2.SelectedNodes
            UltraTree1.Nodes.Add(node.Clone())
            node.Enabled = False

        Next
    End Sub
End Class
